import requests  # type: ignore
import json


# Function to fetch OpenID configuration dynamically
def get_oidc_configuration() -> dict | None:

    with open("config.json", "r") as fid:
        config_json = json.load(fid)

    try:
        # Fetch the well-known configuration
        response = requests.get(
            f"{config_json['issuer_url']}/.well-known/openid-configuration"
        )
        response.raise_for_status()
        config = response.json()

        return {
            "issuer": config["issuer"],
            "client_id": config_json["keycloak_client"],
            "client_secret": config_json["keycloak_secret"],
            "auth_uri": config["authorization_endpoint"],
            "token_uri": config["token_endpoint"],
            "userinfo_uri": config["userinfo_endpoint"],
            "token_introspection_uri": config.get("introspection_endpoint"),
            "jwks_uri": config["jwks_uri"],
        }
    except Exception as e:
        print(f"Error fetching OIDC configuration: {e}")
        return None
